def Author_Lucas(Author: str = 'Nuhil Lucas'):
    '''
    Generate ASCII Art.
    '''
    from pyfiglet import figlet_format
    Result = figlet_format(text=Author, font='starwars', direction='auto', justify='auto', width=10000)
    return Result

def GetTimeStamp(Split: str = '-'):
    '''
    Get Time Stamp.
    '''
    from time import localtime, strftime
    Time_Local: str = localtime()
    Time_Formatted: str = strftime(f'%Y{Split}%m{Split}%d %H{Split}%M{Split}%S', Time_Local)
    return Time_Formatted

def Get_CurrentFrame_Info() -> tuple[str]:  # 获取当前帧信息
    '''
    return (Path_File, Name_Func, FuncLine_Def, FuncLine_Current)
    返回 (文件路径, 函数名, 函数定义的起始行号, 当前执行的行号)
    '''
    from inspect import currentframe
    # 获取当前栈帧
    CurrentFrame = currentframe()
    # 文件名
    Path_File: str = CurrentFrame.f_code.co_filename
    # 函数名
    Name_Func: str = CurrentFrame.f_code.co_name
    # 函数定义的起始行号
    FuncLine_Def: int = CurrentFrame.f_code.co_firstlineno
    # 当前执行的行号 - 即调用currentframe()的行
    FuncLine_Current: int = CurrentFrame.f_lineno
    return (Path_File, Name_Func, FuncLine_Def, FuncLine_Current)



from os.path import exists
from os import mkdir
from pathlib import Path as _Path
from inspect import stack
from src.pylucas.Function.Function import Author_Lucas, GetTimeStamp

class LogManager():
    def __init__(self, OutPutPath_Root) -> None:
        self.TimeStamp: str = ''
        self.OutPutPath_Root: str = OutPutPath_Root
        self.OutPutPath_File: str = OutPutPath_Root

        self.MessageLineBreak: bool = False
        self.LogLimit: list[bool, int] = [True, 10]

        self.Initialize()

    def Initialize(self):
        if not exists(self.OutPutPath_Root): mkdir(self.OutPutPath_Root)

        self.TimeStamp = GetTimeStamp()
        self.OutPutPath_File += rf'\{self.TimeStamp}.txt'
        with open(file=self.OutPutPath_File, mode='w', encoding='utf-8') as file:
            file.write(100 * '-' + '\n')
            file.write(f'{Author_Lucas()}')
            file.write(100 * '-' + '\n')
            file.write(f'Log File Created At {self.TimeStamp}\n\n\n\n\n')
            file.close()
        self.CheckLogLimit()
    
    def SetLogLimit(self, Mode: bool, Limit: int = None):
        if not Limit: self.LogLimit[0] = Mode
        self.LogLimit = [Mode, Limit]

    def SetMessageLB(self, Mode: bool):
        if Mode: self.MessageLineBreak = True
        else: self.MessageLineBreak = False

    def CheckLogLimit(self):
        Path = _Path(self.OutPutPath_Root)
        Files = [f for f in Path.iterdir() if f.is_file() and f.suffix.lower() == '.txt']
        if not Files:
            return
        while (self.LogLimit[0]) and (len(Files) > self.LogLimit[1]):
            OldestFile = min(Files, key=lambda f: f.stat().st_mtime)
            self.LogOutput(LogMessage = f'Deleted Oldest LogFile -> {OldestFile}.')
            OldestFile.unlink()
            Files = [f for f in Path.iterdir() if f.is_file() and f.suffix.lower() == '.txt']

    def LogOutput(self, Module: str = None, Level: str = 'Normal', LogMessage: str = 'Invalid Information', DoPrint: bool = True):
        '''
        Module: str = 'By Auto'
        Level: str = 'Error' | 'Warn' | 'Normal'
        LogMessage: str = 'Invalid Information.'
        DoPrint: bool = True | False
        '''
        if not Module: Module = stack()[1][0].f_globals['__name__']
        TimeStamp = GetTimeStamp()
        Indent: str = ''
        if self.MessageLineBreak: Indent = '\n\t'
        else: Indent = ' '
        if LogMessage[-1] in ('.', '。',): LogMessage = LogMessage[:-1]

        LogText: str = f'{TimeStamp} |-| [Level: <{Level}> | Module: <{Module}>]:{Indent}{LogMessage}.'
        
        if DoPrint:
            print(LogText)
        with open(file=self.OutPutPath_File, mode='a', encoding='utf-8') as file:
            file.write(f'{LogText}\n')


from __future__ import annotations
import tomllib, tomli_w

class ConfigEditor():
    def __init__(self, Path_Toml: str, Data_Toml: dict = {}):
        '''
        Path_Toml: str = 'Temporary' | 'Any Exists Path'
        Data_Toml: dict = {...}
        '''
        self.__Path_Toml: str = ''
        self.__Data_Toml: dict = {}
        self.Temporary: bool = False

        self.Initialize(Path_Toml=Path_Toml, Data_Toml=Data_Toml)

    def Initialize(self, Path_Toml: str, Data_Toml: dict):
        from os.path import exists
        match Path_Toml:
            case 'Temporary':
                self.Temporary = True
                self.Set_Data_Toml(Data_Toml=Data_Toml)
            case _:
                match exists(self.__Path_Toml):
                    case True:
                        self.Load_Toml()
                    case False:
                        self.Save_Toml()

    # Pair --------------------------------------------------
    def Load_Toml(self) -> None:
        if self.Temporary: return
        with open(file=self.__Path_Toml, mode='rb') as File_Toml:
            self.__Data_Toml = tomllib.load(File_Toml)
            File_Toml.close()

    def Save_Toml(self, FiledPath: str = '') -> None:
        if FiledPath and self.Temporary:
            self.Temporary = False
            self.__Path_Toml = FiledPath
        if self.Temporary: return
        with open(file=self.__Path_Toml, mode='wb') as File_Toml:
            tomli_w.dump(self.__Data_Toml, File_Toml)
            File_Toml.close()

    # Pair --------------------------------------------------
    '''
    下面的三个方法中的原变量和新变量不会存在任何引用关系.
    '''
    def Get_Data_Toml(self) -> dict:
        from copy import deepcopy
        return deepcopy(self.__Data_Toml)

    def Set_Data_Toml(self, Data_Toml: dict) -> None:
        '''
        仅在 self.__Data_Toml 为空或任何 bool(self.__Data_Toml)!=True 情况下有效
        不建议使用这个方法
        '''
        from copy import deepcopy
        if self.__Data_Toml:
            return
        self.__Data_Toml = deepcopy(Data_Toml)
        self.Save_Toml()

    def OverWrite_Data(self, Data_Toml: dict):
        '''
        强制覆写 self.__Data_Toml
        不建议使用这个方法
        '''
        from copy import deepcopy
        self.__Data_Toml = deepcopy(Data_Toml)
        self.Save_Toml()

    # Pair --------------------------------------------------
    '''
    下面的三个方法中的原变量和新变量不会存在任何引用关系.
    '''
    def Get_Keys(self, Key_Locate: str = '') -> tuple:
        Key_Locate: list = Key_Locate.split('.')
        Temp_Data: any = self.__Data_Toml
        if Key_Locate[0]:
            for Temp_Key in Key_Locate:
                if type(Temp_Data) != dict: raise KeyError(f'{Key_Locate} -> {Temp_Key}')
                Temp_Data = Temp_Data[Temp_Key]

        return tuple(Temp_Data.keys())

    def Set_Key(self, Key_Locate: str, New_Key: str) -> None:
        pass

    def POP_Key(self, Key_Locate: str):
        Key_Locate: list = Key_Locate.split('.')
        Temp_Data: any = self.__Data_Toml
        for Temp_Key in Key_Locate[:-1]:
            if type(Temp_Data) != dict: raise KeyError(f'{Key_Locate} -> {Temp_Key}')
            Temp_Data = Temp_Data[Temp_Key]
        Temp_Data.pop(Key_Locate[-1])
        self.Save_Toml()

    # Pair --------------------------------------------------
    '''
    下面的三个方法中的原变量和新变量仍然存在引用关系.
    '''
    def Get_Value(self, Key_Locate: str) -> ConfigEditor | any:
        Key_Locate: list = Key_Locate.split('.')
        Temp_Data: any = self.__Data_Toml
        for Temp_Key in Key_Locate:
            if type(Temp_Data) != dict: raise KeyError(f'{Key_Locate} -> {Temp_Key}')
            Temp_Data = Temp_Data[Temp_Key]
        match type(Temp_Data).__name__:
            case 'dict':
                Temp_Data: ConfigEditor = ConfigEditor(Path_Toml='Temporary', Data_Toml=Temp_Data)
            case _:
                pass
        return Temp_Data

    def Set_Value(self, Key_Locate: str, Value: any) -> None:
        '''
        The key path indicated by Key_Locate in the Set_Value requirement may not exist in Data_Toml.
        在 Set_Value 的要求中 Key_Locate 所指示的键路径可以不存在于 Data_Toml 中.
        In the implementation of Set_Value, Set_Value will directly overwrite the Value into the Value of the key-value pair indicated by Key_Locate.
        在 Set_Value 的实现中 Set_Value 会将 Value 直接覆盖到 Key_Locate 所指示的键值对的 Value 中.
        '''
        Key_Locate: list = Key_Locate.split('.')
        Temp_Data: any = self.__Data_Toml
        for Temp_Key in Key_Locate[:-1]:
            if type(Temp_Data) == dict:
                if Temp_Key in Temp_Data:
                    Temp_Data = Temp_Data[Temp_Key]
                else:
                    Temp_Data.update({Temp_Key: {}})
                    Temp_Data = Temp_Data[Temp_Key]
            else:
                raise TypeError(f'Temp_Data: {type(Temp_Data)} = {Temp_Data}')
        Temp_Data.update({Key_Locate[-1]: Value})
        self.Save_Toml()

    def Add_Value(self, Key_Locate: str, Value: any) -> None:
        '''
        The key path indicated by Key_Locate in the requirements of Add_Value must exist in Data_Toml.
        在 Add_Value 的要求中 Key_Locate 所指示的键路径必须存在于 Data_Toml 中.
        In the implementation of Add_Value Add_Value only adds Value to the Value of the key-value pair indicated by Key_Locate.
        在 Add_Value 的实现中 Add_Value 仅会将 Value 添加到 Key_Locate 所指示的键值对的 Value 中.
        '''
        Key_Locate: list = Key_Locate.split('.')
        Temp_Data: any = self.__Data_Toml
        for Temp_Key in Key_Locate:
            if type(Temp_Data) != dict: raise KeyError(f'{Key_Locate} -> {Temp_Key}')
            Temp_Data = Temp_Data[Temp_Key]
        match type(Temp_Data).__name__:
            case 'int':
                raise TypeError('\'int\' object Unable to Add Element')
            case 'float':
                raise TypeError('\'float\' object Unable to Add Element')
            case 'str':
                raise TypeError('\'str\' object Unable to Add Element')
            case 'tuple':
                raise TypeError('\'tuple\' object Unable to Add Element')
            case 'list':
                Temp_Data.append(Value)
            case 'dict':
                if type(Value) != dict: raise TypeError(f'\'dict\' object Unable to Add a Element of {type(Value).__name__}')
                Temp_Data.update(Value)
            case _:
                raise TypeError(f'\'{type(Temp_Data).__name__}\' object UnSupport to AddElement')

        self.Save_Toml()